/*
*
*  $
*  $ FILE     : one.c
*  $ VERSION  : 1
*  $ REVISION : 50
*  $ DATE     : 08-Dec-93 13:15
*  $
*  $ Author   : mvk
*  $
*
**
**
**
** This is a demo program about EGS Gadget.
**
** How EGSGadbox works.
**
** EGSGadBox divides the Window into Boxes (Vertical and Horizontal).
** EGSGadbox need a gadget Context to keep track and informations for
** gadgets. In the beginning there exists
** only one box in Context after which you can add boxes. This means you
** can add a couple of boxes with gadgets on the first box.
**
** The reason to add gadgets into a Context is that if you size your
** window egsgadbox recalculates the gadgets and fits them to the
** new window size. The gadgets are then fully scalable.
**
** The example function My_CreateGads() creates a gadget
** in a horizontal or vertical box
**
**    Window
**   #######################
**   #                     #
**   #     Gadget          #
**   #      ++++           #
**   #      +  +           #
**   #      ++++           #
**   #                     #
**   #######################
**
** If you add 3 gadgets in a horizontal box the
** following window contens is shown:
**
**    Window
**   #######################
**   #                     #
**   #     Gadgets         #
**   #   ++++ ++++ ++++    #
**   #   +1 + +2 + +3 +    #
**   #   ++++ ++++ ++++    #
**   #                     #
**   #######################
**
**     in a vertical box
**
**
**    Window
**   #######################
**   #                     #
**   #      Gadgets        #
**   #        ++++         #
**   #        +1 +         #
**   #        ++++         #
**   #        ++++         #
**   #        ++++         #
**   #        +2 +         #
**   #        ++++         #
**   #        +3 +         #
**   #        ++++         #
**   #                     #
**   #######################
**
**
**   Please look in the file two.c after you have understood
**   the c-source.
**
**
**  (c) by VIONA-Development 92/93
**
*/

/*
**
** These functions are for all examples the same
**
*/

#include "Includes.c"
#include "Global.h"
#include "Open_Close.c"
#include "Eventloop.c"

/**/


/*
**
** This function creates the Gadget using the egsgadbox.library functions
**
*/

EI_WindowPtr My_CreateGads(EI_WindowPtr Window)
{
     EB_GadBoxPtr   root;

     /*
     **
     **  Create Gadget ( CadContext GadCon see global.h )
     **
     */

      if( (GadCon=EB_CreateGadContext(NULL,NULL,-1,-1)) == NULL)
	  myCleanExit(ERR_NOMEM);


	/*
	** Create a Horizontal or Vertical box
	*/
			      /* uncomment only one of
				 the next lines */
	root = EB_CreateHorizBox( GadCon );
//        root = EB_CreateVertiBox( GadCon );

       if( root == NULL )
	  myCleanExit(ERR_NOMEM);

	/*
	**  Add in the Horiz Box a MarkGadget (EI_BoolGadget)
	*/

	/*
	** You could also copy the line to have more in the HorizBox
	*/

	EB_AddLastSon( root, EB_CreateCheckMarkGadget(GadCon,MY_CHECK)     );


	/* Uncomment to add this gadget  */
//        EB_AddLastSon( root, EB_CreateCheckMarkGadget(GadCon,MY_CHECK+1) );
	/* Uncomment to add this gadget  */
//        EB_AddLastSon( root, EB_CreateCheckMarkGadget(GadCon,MY_CHECK+2) );

	/*
	** Set size for the Context
	**
	** Normaly you do not need to call this function, if
	** you have more Gadget. But in this small example
	** we need it to put the gadget in the middle of the
	** window. Without this function the gadget appears in
	** left top corner of the window.
	**
	*/

	root = EB_NewMinHeight( root,100 );
	root = EB_NewMinWidth(  root,100 );

	/*
	**
	** ProcessGadBoxes calculates everything for the gadget(s) (positions)
	**
	*/

	if( EB_ProcessGadBoxes(GadCon,root) )
	{
	  /*
	  **  Filling in the EI_NewWindow structure
	  */

	  /*
	  **  The we open the window with the Context
	  */

	  if( Window == NULL )
	    {
						  /* Add more flags to the window */
	      GadCon->NewWin->Title = "One";

	      GadCon->NewWin->IDCMPFlags          = EI_iCLOSEWINDOW  |
						    EI_iGADGETUP;

	      GadCon->NewWin->Bordef.SysGadgets  |= ( EI_WINDOWCLOSE );

	      Window = EI_OpenWindow( GadCon->NewWin );

	    }

	  return(Window);
	}
    return(NULL);
}



int main(int argc, char *argv[])
{
     /*
     **   Open Libraries
     */

     if ( InitLibraries(OpenStruct) == FALSE )
	  myCleanExit( ERR_LIBS );

	  Window=My_CreateGads(Window);

	  if( Window )
	    {
		 /*
		 **
		 **  Handels IDCMP events (like Commodore)
		 **
		 */

		 HandleEvents( Window );

	     }else{ myCleanExit( ERR_WINDOW );}

      myCleanExit( NULL ) ;
}



