/*
*  $
*  $ FILE     : radio.c
*  $ VERSION  : 1
*  $ REVISION : 9
*  $ DATE     : 08-Dec-93 14:30
*  $
*  $ Author   : mvk
*  $
*
*/

/*

PS: It was a rainy day ! So, I worte somethink up !

This document describes an overview to the use of the gbradio.library.

 Let's say you want 5 radio gadgets for the user to choose from.
They're to be named one, two, three, four, and five respectivly.

 Radio gadgets can be made unselectable if you wish. This is usefull
to keep the user from selecting a choice that at the moment isn't
logical. (ie. using the spare page as your fill mode if there is
currently no spare page open).
 In this case we'll make gadget 2 unselectable if the user resizes
the window (kind of strange, but it will demonstrate the point).

 To unselect a radio, you call EGB_OffRadioElems (window,gadget,long)
   The window is your current window. It will update the window as
   soon as you call the function. (If you give a NULL for a window,
   it won't try to update any window, only the next time the window
   is refreshed, it will update.)
   The gadget is the actual Radio gadget. You find this with
   EB_FindGadget (GadCon->First,GadCon->Num,0x10), where 0x10 is the
   actual ID of the gadget you gave in EB_CreateRadioGadget ().
   The long int in EGB_OffRadioElems is a 32 bit number which represents
   which radio gadget (up to 32) is to be unselectable. The first
   gadget is the least significant bit in the long word. The second
   gadget is the next least significant and so on.
     For example, to unselect the first gadget - 1
		  to unselect the secon gadget - 2
		  to unselect the fith  gadget - 16
     This way, if you later add another gadget, it won't be necessary
     to change any of the previous long int assignments.

  Just for fun, lets have all radio gadgets be selectable if the user ever
  presses radio gadget 5. Now we call EGB_OnRadioElems (window,gadget,long)
  the same way, but pass a 0xFFFFFFFF, to say we want all radios selectable.

    It is also possible to change the current selected radio button to a different
    button. Say the user changed the same option through a menu instead, you
    would then want to reflect those changes in your radio gadget.
    Use EGB_ModifyRadioGadget (window,gadget,data) where data is the number of the
    button you want selected. (In this case, 0-4).

    Related Files: gbradio.doc, gbradio.h <- Our radio gadget
		   egsintui.doc, egsintui.h  <- window stuff
		   egsgadbox.doc, egsgadbox.h <- The actual gadbox
 main.c
***/

#include <stdio.h>
#include <stdlib.h>
#include <exec/types.h>
#include <dos/dos.h>
#include <exec/memory.h>
#include <proto/dos.h>
#include <proto/exec.h>

#include <egs/egs.h>
#include <egs/egsintui.h>
#include <egs/egsgadbox.h>
#include <egs/egb/gbradio.h>
#include <egs/proto/all.h>

struct Library *EGSIntuiBase=NULL;  // EGSIntuition
struct Library *EGBRadioBase=NULL;  // radio
struct Library *EGBBase=NULL;       // gadbox

EB_GadContext GadCon;
EI_WindowPtr window;

EI_WindowPtr RadioWindow (EI_WindowPtr win);
void Clean_Exit (void);

void main (void) {

   EI_EIntuiMsgPtr msg;
   EI_GadgetPtr tgad;
   int class;
   struct EI_Gadget *iaddress;

   if (!(EGSIntuiBase = OpenLibrary ("egsintui.library",0))) {
     printf ("Unable open egsintui.library.\n");
     Clean_Exit ();
   }
   if (!(EGBRadioBase = OpenLibrary ("egb/gbradio.library",0))) {
     printf ("Unable open egbradio.library.\n");
     Clean_Exit ();
   }
   if (!(EGBBase = OpenLibrary ("egsgadbox.library",0))) {
     printf ("Unable open egsgadbox.library.\n");
     Clean_Exit ();
   }

   if (!(window = RadioWindow (NULL))) {
     printf ("Unable to open Window.\n");
     Clean_Exit ();
   }

   for (;;) {  // Forever

     Wait (1<<window->UserPort->mp_SigBit);  // wait for our signals

     while ( (msg=(struct EI_EIntuiMsg *) GetMsg (window->UserPort)) != NULL)
     {

       class = msg->Class;
       iaddress = (struct EI_Gadget *)msg->IAddress;
       switch (class) {

	 case EI_iCLOSEWINDOW:
       ReplyMsg ( (struct Message *)msg);
	    Clean_Exit ();
	 break;
			      // User has begun to resize window
	 case EI_iSIZEVERIFY:
	     EI_LockIntuition ();
	     EI_RemoveGList (window,GadCon->First, GadCon->Num);
	     EB_DeleteGadContext (GadCon);
	     EI_UnlockIntuition ();
       ReplyMsg ( (struct Message *)msg);
	 break;

	 case EI_iNEWSIZE:
	     EI_LockIntuition ();
	     window = RadioWindow (window);
	     EI_AddGList (window,GadCon->First,GadCon->Num);
	     EI_UnlockIntuition ();
			      // Now make radio button 2 unselectable
	     tgad = EB_FindGadget (GadCon->First,GadCon->Num,0x10);
	     EGB_OffRadioElems (window,tgad,0x00000002);
	     EI_RefreshGadget (window,tgad);
       ReplyMsg ( (struct Message *)msg);
	  break;

	  case EI_iGADGETDOWN:  // User has pressed a radio gadget
	     switch ( ((EGB_RadioGadgetPtr)iaddress)->Selected) {
	       case 0: printf ("First radio hit.\n"); break;
	       case 1: printf ("Second radio hit.\n"); break;
	       case 2: printf ("Third radio hit.\n"); break;
	       case 3: printf ("Fourth radio hit.\n"); break;
	       case 4:        // If user presses fifth, make all selectable
		  printf ("Fith radio hit.\n");
		  EGB_OnRadioElems (window,iaddress,0xFFFFFFFF);
       ReplyMsg ( (struct Message *)msg);
	       break;
	     }
	  break;

       }
     }
   }
 }

EI_WindowPtr RadioWindow (EI_WindowPtr win) {

   EB_GadBoxPtr root;
   EB_StrArray radiogads =
	       {"One","Two","Three","Four","Five",NULL};

   GadCon = EB_CreateGadContext (NULL,NULL,-1,-1);

   root = EB_CreateHorizBox (GadCon);
   EB_AddLastSon (root,EB_CreateHorizFill (GadCon,0,0));
   EB_AddLastSon (root,EGB_CreateRadioGadget (GadCon,&radiogads,20,0x10));
   EB_AddLastSon (root,EB_CreateHorizFill (GadCon,0,0));

   root = EB_CreateMasterWindow (GadCon,win,root);
   if (EB_ProcessGadBoxes (GadCon,root)) {
    if (win==NULL) {
      GadCon->NewWin->IDCMPFlags = EI_iCLOSEWINDOW | EI_iSIZEVERIFY |
				   EI_iNEWSIZE | EI_iGADGETUP |
				   EI_iGADGETDOWN;
      GadCon->NewWin->Bordef.SysGadgets |=
				      (EI_WINDOWCLOSE)|(EI_WINDOWSIZE);
      GadCon->NewWin->Flags |= EI_SIZEBBOTTOM;
      return (EI_OpenWindow (GadCon->NewWin));
    }
    else {
      return (win);
    }
  }
  else
    return NULL;
}

void Clean_Exit (void) {

    if (window)
      EI_CloseWindow (window);
    if (GadCon)
      EB_DeleteGadContext (GadCon);
    if (EGBBase)
      CloseLibrary (EGBBase);
    if (EGBRadioBase)
      CloseLibrary (EGBRadioBase);
    if (EGSIntuiBase)
      CloseLibrary (EGSIntuiBase);

    exit (0);
  }
